Imports System
Imports System.Text
Imports System.IO
Imports System.Net
Imports System.Net.Sockets

Public Class cTelnetClient

    Public m_objSocket As Socket
    Public m_nClientListIndex As Integer

    Private m_objTelnetRequest As New cTelnetRequest

    Private m_sLoginTelnet As String
    Private m_nAuthStep As Integer

    Public Const AUTH_LOGIN As Integer = 100
    Public Const AUTH_PASSWORD As Integer = 101
    Public Const AUTH_DONE As Integer = 102

    Private Const RECV_BUFFER_SIZE As Long = 512
    Private m_sAsyncBuffer(RECV_BUFFER_SIZE) As Byte

    Private Shared m_objLockStatment As Object = New Object

    Sub New()
        m_nClientListIndex = -1
        m_nAuthStep = AUTH_LOGIN
    End Sub

    Private Sub OnReceive(ByVal ar As IAsyncResult)

        SyncLock (Me)

            Try

                If m_objSocket Is Nothing Then
                    Exit Sub
                End If

                Dim nRead As Integer = m_objSocket.EndReceive(ar)

                If nRead = 0 Then
                    OnDisconnect()
                    Exit Sub
                End If

                If ReceiveData(Encoding.ASCII.GetString(m_sAsyncBuffer, 0, nRead)) = False Then
                    Exit Sub
                End If

                m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, New AsyncCallback(AddressOf OnReceive), 0)

            Catch e As SocketException
                OnDisconnect()
            End Try

        End SyncLock

    End Sub

    Public Sub OnConnected()

        SendText("Welcome to VaxTele!!" + vbCrLf + vbCrLf + vbCrLf + vbCrLf + "Login:")
        m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, New AsyncCallback(AddressOf OnReceive), 0)

    End Sub

    Public Sub OnDisconnect()

        CloseSocket()
        cTelnetServer.m_objTelnetServer.OnClientDisconnect(Me)

    End Sub

    Private Function OnRequest(ByVal sCommand As String, ByRef objParamList As ArrayList) As Boolean

        SyncLock (Me)

            If String.Compare(sCommand, "BYE", True) = 0 Then

                OnDisconnect()
                OnRequest = False
                Exit Function

            End If

            cServerSIP.m_objServerSIP.OnTelnetCommand(sCommand, objParamList, Me)

        End SyncLock

        OnRequest = True

    End Function

    Public Sub OnSuccessToAuth()
        SendText(vbCrLf + "Success!!" + vbCrLf + vbCrLf)
        SendText("VaxTele:>")
    End Sub

    Public Sub OnFailToAuth()
        SendText(vbCrLf + "Fail!!" + vbCrLf + vbCrLf)

        m_nAuthStep = AUTH_LOGIN
        SendText("Login:")
    End Sub

    Public Sub SendCommand(ByVal sCommand As String, ByRef objParamList As ArrayList)

        Dim sToData As String = sCommand

        If Not objParamList Is Nothing Then

            Dim nCount As Integer
            For nCount = 0 To objParamList.Count - 1
                sToData = sToData + " " + objParamList.Item(nCount)
            Next

        End If

        sToData = sToData + vbCrLf
        m_objSocket.Send(Encoding.UTF8.GetBytes(sToData), 0, sToData.Length, SocketFlags.None)

    End Sub

    Public Sub SendText(ByVal sText As String)
        Dim msg As Byte() = Encoding.UTF8.GetBytes(sText)
        m_objSocket.Send(msg, 0, msg.Length, SocketFlags.None)
    End Sub

    Private Sub OnAuthrization(ByVal sData As String)

        If m_nAuthStep = AUTH_LOGIN Then

            m_nAuthStep = AUTH_PASSWORD
            m_sLoginTelnet = sData

            SendText("Password:")

        ElseIf m_nAuthStep = AUTH_PASSWORD Then

            m_nAuthStep = AUTH_DONE

            If cTelnetServer.m_objTelnetServer.IsAuth(m_sLoginTelnet, sData) = True Then
                OnSuccessToAuth()
            Else
                OnFailToAuth()
            End If

            m_sLoginTelnet = ""

        End If

    End Sub

    Private Function ReceiveData(ByVal sRecvData As String) As Boolean

        m_objTelnetRequest.FillRecvBuffer(sRecvData, Me)

        While True

            Dim sRequest As String = m_objTelnetRequest.FindRequest(Me)
            If sRequest.Length = 0 Then
                Exit While
            End If

            If m_nAuthStep <> AUTH_DONE Then
                OnAuthrization(sRequest)
                Exit While
            End If

            Dim sCommand As String = ""
            Dim objParamList As New ArrayList

            If m_objTelnetRequest.ParseRequest(sRequest, sCommand, objParamList) = False Then
                Exit While
            End If

            If OnRequest(sCommand, objParamList) = False Then
                ReceiveData = False
                Exit Function
            End If

        End While

        ReceiveData = True

    End Function

    Public Sub CloseSocket()

        If Not m_objSocket Is Nothing Then

            SyncLock (Me)

                m_objSocket.Close()
                m_objSocket = Nothing

            End SyncLock

        End If

    End Sub

End Class
