// QueueList.cpp: implementation of the CQueueList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "QueueList.h"
#include "Registry.h"
#include "CallList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define DETECT_DIGIT_TIME_START		1
#define DETECT_DIGIT_TIME_UP		2

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CQueueList::CQueueList()
{
	m_pQueueListTEXT = new CMapQueueStrToPtr;
	m_pQueueListINT = new CMapQueueULongToPtr;

	m_lQueueIdCount = 0;
}

CQueueList::~CQueueList()
{
	DeleteAll();
	
	delete m_pQueueListTEXT;
	delete m_pQueueListINT;
}

void CQueueList::UnLoadQueue()
{
	DeleteAll();
}

ULONG CQueueList::ChooseQueueId()
{
	while(TRUE)
	{
		m_lQueueIdCount++;

		if(FindQueue(m_lQueueIdCount) == NULL) break;
	}

	return m_lQueueIdCount;
}

void CQueueList::LoadQueue()
{
	CRegistry Registry;
	
	CStringList objQueueList;
	Registry.EnumQueue(&objQueueList);
	
	while(!objQueueList.IsEmpty())
	{
		CString sQueueName = objQueueList.RemoveHead();
		
		CQueueReg objQueueReg;
		if(!Registry.GetQueue(sQueueName, &objQueueReg))
		{
			char sStatus[128] = "\0";
			::_stprintf(sStatus, "Unable to load queue [ %s ] information from registry.", sQueueName);

			CServerSIP::m_pServerSIP->LogError(sStatus);
			continue;
		}

		Add(sQueueName, objQueueReg.m_sPlayWave, objQueueReg.m_sPressDigitPlayWave, &objQueueReg.m_objQueueOPList, &objQueueReg.m_objPressDigitList, &objQueueReg.m_objPressDigitUserList);
	}
}


BOOL CQueueList::AddQueue(LPCTSTR sQueueName, LPCTSTR sQueuePlayWave, LPCTSTR sDigitPressPlayWave, CStringList* pAgentList, CStringList* pPressDigitList, CStringList* pUserDigitList)
{
	CQueueReg objQueueReg;

	objQueueReg.m_sPlayWave           = sQueuePlayWave;
	objQueueReg.m_sPressDigitPlayWave = sDigitPressPlayWave;
	
	int nCount;
	
	for(nCount = 0; nCount < pAgentList->GetCount(); nCount++)
		objQueueReg.m_objQueueOPList.AddTail(pAgentList->GetAt(pAgentList->FindIndex(nCount)));
		
	for(nCount = 0; nCount < pPressDigitList->GetCount(); nCount++)
		objQueueReg.m_objPressDigitList.AddTail(pPressDigitList->GetAt(pPressDigitList->FindIndex(nCount)));
	
	for(nCount = 0; nCount < pUserDigitList->GetCount(); nCount++)
		objQueueReg.m_objPressDigitUserList.AddTail(pUserDigitList->GetAt(pUserDigitList->FindIndex(nCount)));
	
	CRegistry objReg;
	
	if(!objReg.SetQueue(sQueueName, &objQueueReg))
		return FALSE;
	
	Add(sQueueName, sQueuePlayWave, sDigitPressPlayWave, pAgentList, pPressDigitList, pUserDigitList);

	return TRUE;
}

BOOL CQueueList::RemoveQueue(LPCTSTR sQueueName)
{
	CRegistry objReg;
	
	if(!objReg.RemoveQueue(sQueueName))
		return FALSE;

	Remove(sQueueName);

	return TRUE;
}


BOOL CQueueList::FindFreeUserId(CStringList* pListOP, CString& sFreeUserId)
{
	for(POSITION pos = pListOP->GetHeadPosition(); pos != NULL;)
	{
		LPCTSTR sUserId = pListOP->GetNext(pos);

		if(CServerSIP::m_pServerSIP->m_objUserList.GetCallCount(sUserId) == 0)
		{
			sFreeUserId = sUserId;
			return TRUE;
		}
	}

	return FALSE;
}

void CQueueList::PlayQueueWaveToCall(CCallData* pCallData, ULONG lQueueWaveId, BOOL bLoopPlay)
{
	if(lQueueWaveId != -1)
		CServerSIP::m_pServerSIP->PlayWaveStartToCall(pCallData, lQueueWaveId, bLoopPlay);
}

BOOL CQueueList::AddCallToQueue(CCallData* pCallData)
{
	CQueueData* pQueueData = FindQueue(pCallData->m_lQueueId);
	if(pQueueData == NULL) return FALSE;
	
	pQueueData->m_objQueueCallList.AddTail(pCallData);
	pCallData->m_bDigitDetect = pQueueData->m_bIsDigitDetect;

	UpdateCall(pCallData, pQueueData);
	return TRUE;
}


void CQueueList::UpdateCall(CCallData* pCallData, CQueueData* pQueueData)
{
	if(pCallData->m_bDigitDetect)
	{
		CServerSIP::m_pServerSIP->EnableDetectDTMF(pCallData);
					
		PlayQueueWaveToCall(pCallData, pQueueData->m_lQueueDigitDetectPlayWaveId, FALSE);
	}
	else
	{
		PlayQueueWaveToCall(pCallData, pQueueData->m_lQueuePlayWaveId, TRUE);
						
		if(pQueueData->m_objQueueCallList.GetCount() == 1)
			DistributeCall(pQueueData);
	}
}


void CQueueList::ProcessQueueTick()
{
	ULONG lKey;
	CQueueData* pQueueData;

	for(POSITION pos = m_pQueueListINT->GetStartPosition(); pos != NULL;)
	{
		m_pQueueListINT->GetNextAssoc( pos, lKey, pQueueData);
		
		DistributeCall(pQueueData);
	}
}

void CQueueList::DistributeCall(CQueueData* pQueueData)
{
	for(POSITION pos = pQueueData->m_objQueueCallList.GetHeadPosition(); pos != NULL;)
	{
		POSITION CurrentPOS = pos;
		CCallData* pQueueCallData = pQueueData->m_objQueueCallList.GetNext(pos);
		
		if(pQueueCallData->m_bDigitDetect)
		{
			if(!DistributeCallToDigitPressUser(pQueueData, pQueueCallData))
				continue;
		}
		else
		{
			if(!DistributeCallToQueueAgent(pQueueData, pQueueCallData))
				return;
				
		}

		pQueueData->m_objQueueCallList.RemoveAt(CurrentPOS);
	}
}

BOOL CQueueList::DistributeCallToQueueAgent(CQueueData* pQueueData, CCallData* pQueueCallData)
{
	CString sFreeAgentUserId;
	if(!FindFreeUserId(&pQueueData->m_objAgentList, sFreeAgentUserId)) return FALSE;

	CCallData* pDialCallData = CServerSIP::m_pServerSIP->DialCallToUser(pQueueCallData, sFreeAgentUserId);
	if(pDialCallData == NULL) return FALSE;
	
	PlayDialToneToCall(pQueueCallData);

	return TRUE;
}

BOOL CQueueList::DistributeCallToDigitPressUser(CQueueData* pQueueData, CCallData* pQueueCallData)
{
	if(pQueueCallData->m_nDigitPRESS == DETECT_DIGIT_TIME_START)
	{
		pQueueCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_UP;
		return FALSE;
	}
	
	if(pQueueCallData->m_nDigitPRESS == DETECT_DIGIT_TIME_UP)
	{
		if(DialCallToDigitPRESS(pQueueData, pQueueCallData))
			return TRUE;
				
		CServerSIP::m_pServerSIP->DisableDetectDTMF(pQueueCallData);
		UpdateCall(pQueueCallData, pQueueData);
	}

	return FALSE;
}

BOOL CQueueList::DialCallToDigitPRESS(CQueueData* pQueueData, CCallData* pQueueCallData)
{
	if(pQueueCallData->m_sDigitPRESS.IsEmpty())
		return FALSE;

	CString sDigitPressUSER;

	if(FindUserForDigitPRESS(pQueueData, pQueueCallData->m_sDigitPRESS, sDigitPressUSER))
	{
		CCallData* pDialCallData = CServerSIP::m_pServerSIP->DialCallToUser(pQueueCallData, sDigitPressUSER);
		
		if(pDialCallData != NULL) 
		{
			CServerSIP::m_pServerSIP->DisableDetectDTMF(pQueueCallData);
			PlayDialToneToCall(pQueueCallData);

			return TRUE;
		}
	}

	return FALSE;
}

BOOL CQueueList::FindUserForDigitPRESS(CQueueData* pQueueData, LPCTSTR sDigitPRESS, CString& sDigitPressUSER)
{
	return pQueueData->m_objDigitDetectList.Lookup(sDigitPRESS, sDigitPressUSER);
}

void CQueueList::OnPressDigitDTMF(CCallData* pCallData, char sDigit)
{
	CServerSIP::m_pServerSIP->PlayWaveStopToCall(pCallData);
	
	pCallData->m_sDigitPRESS += sDigit;
	pCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_START;
}

void CQueueList::OnPlayWaveDoneToCall(CCallData* pCallData)
{
	pCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_START;
}

void CQueueList::PlayDialToneToCall(CCallData* pCallData)
{
	long lDialToneSessionId = CServerSIP::m_pServerSIP->m_objServerData.GetDialToneSessionId();
		
	if(lDialToneSessionId != -1) 
		CServerSIP::m_pServerSIP->AddCallToSession(pCallData, lDialToneSessionId);
}

void CQueueList::RemoveQueueCall(CCallData* pCallData)
{
	CQueueData* pQueueData = FindQueue(pCallData->m_lQueueId);
	if(pQueueData == NULL) return;

	POSITION pos = pQueueData->m_objQueueCallList.Find(pCallData);
	
	if(pos != NULL)
		pQueueData->m_objQueueCallList.RemoveAt(pos);
}

CQueueData* CQueueList::FindQueue(ULONG lQueueId)
{
	CQueueData* pQueueData;
	if(!m_pQueueListINT->Lookup(lQueueId, pQueueData))
		return NULL;
	
	return pQueueData;
}

CQueueData* CQueueList::FindQueue(CString sQueueName)
{
	sQueueName.MakeLower();

	CQueueData* pQueueData;
	if(!m_pQueueListTEXT->Lookup(sQueueName, pQueueData))
		return NULL;
	
	return pQueueData;
}

void CQueueList::DeleteAll()
{
	ULONG lKey;
	CQueueData* pQueueData;

	for(POSITION pos = m_pQueueListINT->GetStartPosition(); pos != NULL;)
	{
		m_pQueueListINT->GetNextAssoc( pos, lKey, pQueueData);
						
		CServerSIP::m_pServerSIP->PlayWaveUnLoadFile(pQueueData->m_lQueuePlayWaveId);
		CServerSIP::m_pServerSIP->PlayWaveUnLoadFile(pQueueData->m_lQueueDigitDetectPlayWaveId);
		
		delete pQueueData;
	}

	m_pQueueListINT->RemoveAll();
	m_pQueueListTEXT->RemoveAll();
}


void CQueueList::Remove(CString sQueueName)
{
	CQueueData* pQueueData = FindQueue(sQueueName);
	if(pQueueData == NULL) return;
		
	for(POSITION pos = pQueueData->m_objQueueCallList.GetHeadPosition(); pos != NULL;)
	{
		CCallData* pQueueCallData = pQueueData->m_objQueueCallList.GetNext(pos);
		CServerSIP::m_pServerSIP->CloseCall(pQueueCallData);
	}

	m_pQueueListINT->RemoveKey(pQueueData->m_lQueueId);
	
	sQueueName.MakeLower();
	m_pQueueListTEXT->RemoveKey(sQueueName);

	delete pQueueData;
}


BOOL CQueueList::Add(CString sQueueName, LPCTSTR sQueuePlayWave, LPCTSTR sDigitPressPlayWave, CStringList* pAgentList, CStringList* pPressDigitList, CStringList* pUserDigitList)
{
	Remove(sQueueName);
	
	LONG lPlayWaveId = -1, lDigitPressPlayWaveId = -1;
		
	if(::lstrlen(sQueuePlayWave) != 0)
	{
		CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sQueuePlayWave) + " ] wave file....");
		
		lPlayWaveId = CServerSIP::m_pServerSIP->PlayWaveLoadFile(sQueuePlayWave);
		
		if(lPlayWaveId == -1)	
		{
			CServerSIP::m_pServerSIP->LogError("");
			CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sQueuePlayWave) + " ] wave file fail.");
		}
		else
		{
			CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sQueuePlayWave) + " ] wave file done.");
		}
	}

	if(::lstrlen(sDigitPressPlayWave) != 0)
	{
		CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sDigitPressPlayWave) + " ] wave file....");

		lDigitPressPlayWaveId = CServerSIP::m_pServerSIP->PlayWaveLoadFile(sDigitPressPlayWave);
		
		if(lDigitPressPlayWaveId == -1) 
		{
			CServerSIP::m_pServerSIP->LogError("");
			CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sDigitPressPlayWave) + " ] wave file fail.");
		}
		else
		{
			CServerSIP::m_pServerSIP->LogStatus("Loading [ " + CString(sDigitPressPlayWave) + " ] wave file done.");
		}
	}
	
	CQueueData* pQueueData = new CQueueData;

	if(pPressDigitList->IsEmpty())
		pQueueData->m_bIsDigitDetect = FALSE;
	else
		pQueueData->m_bIsDigitDetect = TRUE;
	
	while(!pAgentList->IsEmpty())
		pQueueData->m_objAgentList.AddTail(pAgentList->RemoveHead());

	while(!pPressDigitList->IsEmpty())
	{
		CString sDigitPress     = pPressDigitList->RemoveHead();
		CString sDigitPressUSER = pUserDigitList->RemoveHead();
		
		pQueueData->m_objDigitDetectList.SetAt(sDigitPress, sDigitPressUSER);
	}
	
	pQueueData->m_lQueueId			          = ChooseQueueId();

	pQueueData->m_lQueueDigitDetectPlayWaveId = lDigitPressPlayWaveId;
	pQueueData->m_lQueuePlayWaveId		      = lPlayWaveId;

	m_pQueueListINT->SetAt(pQueueData->m_lQueueId, pQueueData);
	
	sQueueName.MakeLower();
	m_pQueueListTEXT->SetAt(sQueueName, pQueueData);

	return TRUE;
}