using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Collections;
using System.Threading;

class cTelnetClient
{
    public Socket m_objSocket = null;
    public int m_nClientListIndex;

    private cTelnetRequest m_objTelnetRequest = new cTelnetRequest();

    private string m_sLoginTelnet;
    private int m_nAuthStep;

    public const int AUTH_LOGIN = 100;
    public const int AUTH_PASSWORD = 101;
    public const int AUTH_DONE = 102;

    private const int RECV_BUFFER_SIZE = 512;
    private byte[] m_sAsyncBuffer = new byte[RECV_BUFFER_SIZE];
        
    private static Object m_objLockStatment = new Object();

    public cTelnetClient()
    {
        m_nClientListIndex = -1;
        m_nAuthStep = AUTH_LOGIN;
    }

    private void OnReceive(IAsyncResult ar)
    {        
        lock (this)
        {
            if (m_objSocket == null)
                return;

            try
            {
                int nRead = m_objSocket.EndReceive(ar);

                if (nRead == 0)
                {
                    OnDisconnect();
                    return;
                }

                if (ReceiveData(Encoding.ASCII.GetString(m_sAsyncBuffer, 0, nRead)) == false)
                    return;

                m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, new AsyncCallback(OnReceive), 0);
            }
            catch (SocketException e)
            {
                OnDisconnect();
            }
        }
    }

    public void OnDisconnect()
    {
        CloseSocket();

        cTelnetClient objTelnetClient = this;
        cTelnetServer.m_objTelnetServer.OnClientDisconnect(ref objTelnetClient);
    }

    public void OnConnected()
    {
        SendText("Welcome to VaxTele!!\r\n\r\n\r\nLogin:");
        m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, new AsyncCallback(OnReceive), 0);
    }

    private Boolean OnRequest(string sCommand, ref ArrayList objParamList)
    {
        lock (m_objLockStatment)
        {
            if (String.Compare(sCommand, "BYE", true) == 0)
            {
                OnDisconnect();
                return false;
            }

            cTelnetClient objTelnetClient = this;
            cServerSIP.m_objServerSIP.OnTelnetCommand(sCommand, ref objParamList, ref objTelnetClient);
        }

        return true;
    }

    public void OnSuccessToAuth()
    {
        SendText("\r\nSuccess!!\r\n\r\n");
	    SendText("VaxTele:>");
    }

    public void OnFailToAuth()
    {
        SendText("\r\nFail!!\r\n\r\n");

	    m_nAuthStep = AUTH_LOGIN;
	    SendText("Login:");
    }

    public void SendCommand(string sCommand, ref ArrayList objParamList)
    {
        String sToData = sCommand;

        if(objParamList != null)
        {
            for(int nCount = 0; nCount < objParamList.Count; nCount++)
                sToData = sToData + " " + objParamList[nCount];
        }

        sToData += "\r\n";
        m_objSocket.Send(Encoding.UTF8.GetBytes(sToData), 0, sToData.Length, SocketFlags.None);
    }

    public void SendText(string sText)
    {
        byte[] msg = Encoding.UTF8.GetBytes(sText);
        m_objSocket.Send(msg, 0, msg.Length, SocketFlags.None);
    }

    private void OnAuthrization(string sData)
    {
        if(m_nAuthStep == AUTH_LOGIN)
        {
            m_nAuthStep = AUTH_PASSWORD;
            m_sLoginTelnet = sData;

            SendText("Password:");
        }
        else if(m_nAuthStep == AUTH_PASSWORD)
        {
            m_nAuthStep = AUTH_DONE;

            if(cTelnetServer.m_objTelnetServer.IsAuth(m_sLoginTelnet, sData) == true)
                OnSuccessToAuth();
            else
                OnFailToAuth();
            
            m_sLoginTelnet = "";
        }
    }
        
    private Boolean ReceiveData(string sRecvData)
    {
        cTelnetClient objTelnetClient = this;

        m_objTelnetRequest.FillRecvBuffer(sRecvData, ref objTelnetClient);

        while(true)
        {
            string sRequest = m_objTelnetRequest.FindRequest(ref objTelnetClient);

            if(sRequest.Length == 0)
                break;

            if (m_nAuthStep != AUTH_DONE)
            {
                OnAuthrization(sRequest);
                break;
            }
            
            string sCommand = "";
            ArrayList objParamList = new ArrayList();

            if(m_objTelnetRequest.ParseRequest(sRequest, ref sCommand, ref objParamList) == false)
                break;
            
            if(OnRequest(sCommand, ref objParamList) == false)
                return false;
        }

        return true;
    }

    public void CloseSocket()
    {
        lock (this)
        {
            if (m_objSocket != null)
            {
                m_objSocket.Close();
                m_objSocket = null;
            }
        }
    }
}

